#include "svc_basic.h"

void svc_basic_auth(thread t, jpacket p, xmlnode query)
{
    mapi m;
    char *username, *password, *resource;

    /* set this packet as the response to the IQ */
    xmlnode_hide_attrib(p->x, "type");
    xmlnode_put_attrib(p->x,"type","result");

    username = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "username")));
    password = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "password")));
    resource = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(query, "resource")));

    if(resource == NULL)
    {
        svc_basic_auth_result(query,"illegal","Invalid Authenticaiton Query");
        return;
    }

    /* do they want to be anonymous? */
    if(username == NULL || password == NULL)
    {
        if(xmlnode_get_tag(svc_basic_config,"anonymous") == NULL)
        {
            svc_basic_auth_result(query,"illegal","Invalid Authenticaiton Query");
            return;
        }

        jdebug(ZONE,"SVC_BASIC: creating anonymous resource");
        if(resource_lookup(NULL,resource) != NULL)
        {
            svc_basic_auth_result(query,"resource","Requested Resource In Use");
            return;
        }

        /* attempt to create a session for the anonymous resource */
        p->from->resource = resource;
        if(!svc_basic_new(t, p->from))
        {
            svc_basic_auth_result(query,"session","Session Creation Failed");
            return;
        }
        xmlnode_hide(xmlnode_get_tag(query, "resource"));
        svc_basic_auth_result(query,"ok","Anonymous Resource Created");
        return;
    }

    /* create a mapi for auth */
    m = mapi_new(M_PHASE_AUTH, p->p, NULL, p);
    m->user = username;
    m->resource = resource;
    m->password = password;

    /* see if we please the gods */
    module_dispatch(m);
    if(m->result == M_HANDLED)
    {
        if(resource_lookup(username, resource) != NULL)
        {
            svc_basic_auth_result(query,"resource","Requested Resource In Use");
            return;
        }

        /* attempt to create a session for these credentials */
        p->from->resource = resource;
        p->from->user = username;
        if(!svc_basic_new(t, p->from))
        {
            svc_basic_auth_result(query,"session","Session Creation Failed");
            return;
        }
        xmlnode_hide(xmlnode_get_tag(query, "username"));
        xmlnode_hide(xmlnode_get_tag(query, "password"));
        xmlnode_hide(xmlnode_get_tag(query, "resource"));
        svc_basic_auth_result(query,"ok",NULL);
    }else if(m->result == M_ERROR){
        svc_basic_auth_result(query,"auth",m->auth_error);
    }else{
        svc_basic_auth_result(query,"auth","Unknown User");
    }
}


void svc_basic_auth_result(xmlnode query, char *type, char *message)
{
    xmlnode result;

    jdebug(ZONE,"SVC_BASIC: auth result %s/%s",type,message);

    xmlnode_hide(xmlnode_get_tag(query, "result"));
    result = xmlnode_insert_tag(query, "result");
    xmlnode_put_attrib(result,"type",type);
    if(message != NULL)
        xmlnode_insert_cdata(result,message,strlen(message));
}

void svc_basic_register(char *host, jpacket p)
{
    mapi m;

    jdebug(ZONE,"SVC_BASIC: registration req %s",host);

    /* call the mapi for registration */
    m = mapi_new(M_PHASE_IQ_HANDLE, p->p, NULL, p);
    m->user = host; /* sneak in the hostname */
    module_dispatch(m);

    /* set this packet as the response to the IQ */
    xmlnode_hide_attrib(p->x, "type");
    xmlnode_put_attrib(p->x,"type","result");

}
