#include "jserver.h"
#include <getopt.h>

/* this file is only for the executable version of the jabber server */

int main (int argc, char** argv)
{
    pth_event_t sigevt;
    sigset_t set;
    int help, sig, i, sz;
    char *etherx, *id, *secret;
    struct option options[] =
    {
        {"etherx",1,0,'e'},
        {"id",1,0,'i'},
        {"secret",1,0,'s'},
        {0,0,0,0}
    };

    etherx = id = secret = NULL;
    help = opterr = 0;
    while((sig = getopt_long(argc,argv,"e:i:s:",options,NULL)) > 0)
    {
        switch(sig)
        {
            case 'e':
                etherx = strdup(optarg);
                break;
            case 'i':
                id = strdup(optarg);
                break;
            case 's':
                secret = strdup(optarg);
				sz = strlen(optarg);
				for(i=0;i<sz;i++)
					optarg[i]='x';
                break;
            default:
                help = 1;
        }
    }
 
    if(help || secret == NULL)
    {
        printf("Usage:\njserver -s secret [-e etherx.host] [-i transport_id]\n");
        exit(0);
    }
 
    if(etherx == NULL)
        etherx = strdup("127.0.0.1");

    if(id == NULL)
        id = strdup("jabber");

    jlib_init(JLIB_LOG_STDERR, NULL);

    jdebug(ZONE,"starting transport %s",id);

    /* start the connection to etherx which will start up our internals */
    if(etherx_init(id,secret,etherx,NS_SERVER,&etherx_receive) < 0)
    {
        printf("Connection to etherx on %s failed, exiting\n",etherx);
        exit(0);
    }

    sigemptyset(&set);
    sigaddset(&set, SIGHUP);
    sigaddset(&set, SIGINT);
    sigaddset(&set, SIGTERM);
    pth_sigmask(SIG_UNBLOCK, &set, NULL);
    sigevt = pth_event(PTH_EVENT_SIGS, &set, &sig);
 
    jdebug(ZONE,"Main loop waiting");
    pth_wait(sigevt);

    jerror("jserver","Received KILL, shutting down");
    shutdown_now();
    pth_yield(NULL); /* one last chance to make sure everyone is done */
    pth_kill();
    return 0;
}
